#!/bin/bash -e
#
# racoon up/down script for Shimo
#

##########################################################################################
# @param String message - The message to log
logMessage()
{
    echo "$(date '+%Y-%m-%d %T') *Shimo: "${@} >> "${SCRIPT_LOG_FILE}"
}

##########################################################################################
# @param String string - Content to trim
trim()
{
    echo ${@}
}

print_variables()
{
    if [ -n "${UNIQUE_IDENTIFIER}" ]; then logMessage "UNIQUE_IDENTIFIER = ${UNIQUE_IDENTIFIER}"; fi
    if [ -n "${LOCAL_ADDR}" ]; then logMessage "LOCAL_ADDR = ${LOCAL_ADDR}"; fi
    if [ -n "${LOCAL_PORT}" ]; then logMessage "LOCAL_PORT = ${LOCAL_PORT}"; fi
    if [ -n "${REMOTE_ADDR}" ]; then logMessage "REMOTE_ADDR = ${REMOTE_ADDR}"; fi
    if [ -n "${REMOTE_PORT}" ]; then logMessage "REMOTE_PORT = ${REMOTE_PORT}"; fi
    if [ -n "${DEFAULT_GW}" ]; then logMessage "DEFAULT_GW = ${DEFAULT_GW}"; fi
    if [ -n "${INTERNAL_ADDR4}" ]; then logMessage "INTERNAL_ADDR4 = ${INTERNAL_ADDR4}"; fi
    if [ -n "${INTERNAL_NETMASK4}" ]; then logMessage "INTERNAL_NETMASK4 = ${INTERNAL_NETMASK4}"; fi
    if [ -n "${INTERNAL_DNS4}" ]; then logMessage "INTERNAL_DNS4 = ${INTERNAL_DNS4}"; fi
    if [ -n "${INTERNAL_DNS4_LIST}" ]; then logMessage "INTERNAL_DNS4_LIST = ${INTERNAL_DNS4_LIST}"; fi
    if [ -n "${INTERNAL_WINS4}" ]; then logMessage "INTERNAL_WINS4 = ${INTERNAL_WINS4}"; fi
    if [ -n "${INTERNAL_WINS4_LIST}" ]; then logMessage "INTERNAL_WINS4_LIST = ${INTERNAL_WINS4_LIST}"; fi
    if [ -n "${INTERNAL_ADDR6}" ]; then logMessage "INTERNAL_ADDR6 = ${INTERNAL_ADDR6}"; fi
    if [ -n "${INTERNAL_NETMASK6}" ]; then logMessage "INTERNAL_NETMASK6 = ${INTERNAL_NETMASK6}"; fi
    if [ -n "${INTERNAL_DNS6}" ]; then logMessage "INTERNAL_DNS6 = ${INTERNAL_DNS6}"; fi
    if [ -n "${INTERNAL_DNS6_LIST}" ]; then logMessage "INTERNAL_DNS6_LIST = ${INTERNAL_DNS6_LIST}"; fi
    if [ -n "${INTERNAL_WINS6}" ]; then logMessage "INTERNAL_WINS6 = ${INTERNAL_WINS6}"; fi
    if [ -n "${INTERNAL_WINS6_LIST}" ]; then logMessage "INTERNAL_WINS6_LIST = ${INTERNAL_WINS6_LIST}"; fi
    if [ -n "${DEFAULT_DOMAIN}" ]; then logMessage "DEFAULT_DOMAIN = ${DEFAULT_DOMAIN}"; fi
    if [ -n "${INTERFACE}" ]; then logMessage "INTERFACE = ${INTERFACE}"; fi
    if [ -n "${SPLIT_INCLUDE}" ]; then logMessage "SPLIT_INCLUDE = ${SPLIT_INCLUDE}"; fi
    if [ -n "${SPLIT_LOCAL}" ]; then logMessage "SPLIT_LOCAL = ${SPLIT_LOCAL}"; fi
    logMessage "ARG_DEFAULT_ROUTE = ${ARG_DEFAULT_ROUTE}"
    logMessage "ARG_PREPEND_DOMAIN_NAME = ${ARG_PREPEND_DOMAIN_NAME}"
    logMessage "ARG_FLUSH_DNS_CACHE = ${ARG_FLUSH_DNS_CACHE}"
    logMessage "ARG_OVERWRITE_NETCONF = ${ARG_OVERWRITE_NETCONF}"
    logMessage "ARG_RESET_PRIMARY_INTERFACE_ON_DISCONNECT = ${ARG_RESET_PRIMARY_INTERFACE_ON_DISCONNECT}"
}

##########################################################################################
# @param String[] dnsServers - The name servers to use
# @param String domainName - The domain name to use
# @param \optional String[] winsServers - The SMB servers to use
#
# Throughout this routine:
#            MAN_ is a prefix for manually set parameters
#            DYN_ is a prefix for dynamically set parameters (by a "push", config file, or command line option)
#            CUR_ is a prefix for the current parameters (as arbitrated by OS X between manual and DHCP data)
#            FIN_ is a prefix for the parameters we want to end up with
#            SKP_ is a prefix for an empty string or a "#" used to control execution of statements that set parameters in scutil
#
#            DNS_SA is a suffix for the ServerAddresses value in a System Configuration DNS key
#            DNS_SD is a suffix for the SearchDomains   value in a System Configuration DNS key
#            DNS_DN is a suffix for the DomainName      value in a System Configuration DNS key
#
#            SMB_NN is a suffix for the NetBIOSName   value in a System Configuration SMB key
#            SMB_WG is a suffix for the Workgroup     value in a System Configuration SMB key
#            SMB_WA is a suffix for the WINSAddresses value in a System Configuration SMB key
#
# So, for example, MAN_SMB_NN is the manually set NetBIOSName value (or the empty string if not set manually)
modify_dns()
{
    set +e # "grep" will return error status (1) if no matches are found, so don't fail on individual errors

    PSID="$( scutil <<-EOF |
        open
        show State:/Network/Global/IPv4
        quit
EOF
grep PrimaryService | sed -e 's/.*PrimaryService : //'
)"

    set -e # resume abort on error

    MAN_DNS_CONFIG="$( scutil <<-EOF |
        open
        show Setup:/Network/Service/${PSID}/DNS
        quit
EOF
sed -e 's/^[[:space:]]*[[:digit:]]* : //g' | tr '\n' ' '
)"

    MAN_SMB_CONFIG="$( scutil <<-EOF |
        open
        show Setup:/Network/Service/${PSID}/SMB
        quit
EOF
sed -e 's/^[[:space:]]*[[:digit:]]* : //g' | tr '\n' ' '
)"
    CUR_DNS_CONFIG="$( scutil <<-EOF |
        open
        show State:/Network/Global/DNS
        quit
EOF
sed -e 's/^[[:space:]]*[[:digit:]]* : //g' | tr '\n' ' '
)"

    CUR_SMB_CONFIG="$( scutil <<-EOF |
        open
        show State:/Network/Global/SMB
        quit
EOF
sed -e 's/^[[:space:]]*[[:digit:]]* : //g' | tr '\n' ' '
)"

# Set up the DYN_... variables to contain what is asked for (dynamically, by a 'push' directive, for example)

    declare -a vDNS=(${INTERNAL_DNS4_LIST[@]})
    declare -a vSMB=(${INTERNAL_WINS4_LIST[@]})

    if [ ${#vDNS[*]} -eq 0 ] ; then
        readonly DYN_DNS_SA=""
    else
        readonly DYN_DNS_SA="$(trim "${vDNS[*]}")"
    fi
    
    if [ ${#vSMB[*]} -eq 0 ] ; then
        readonly DYN_SMB_WA=""
    else
        readonly DYN_SMB_WA="$(trim "${vSMB[*]}")"
    fi

    DYN_DNS_DN="$DEFAULT_DOMAIN"
    
    # The variables
    #     DYN_DNS_SD
    #     DYN_SMB_WG
    #     DYN_SMB_NN
    # are left empty. There isn't a way for Racoon to set them.

    logMessage "DEBUG:"
    logMessage "DEBUG: MAN_DNS_CONFIG = ${MAN_DNS_CONFIG}"
    logMessage "DEBUG: MAN_SMB_CONFIG = ${MAN_SMB_CONFIG}"
    logMessage "DEBUG:"
    logMessage "DEBUG: CUR_DNS_CONFIG = ${CUR_DNS_CONFIG}"
    logMessage "DEBUG: CUR_SMB_CONFIG = ${CUR_SMB_CONFIG}"
    logMessage "DEBUG:"
    logMessage "DEBUG:"
    logMessage "DEBUG: DYN_DNS_DN = ${DYN_DNS_DN}; DYN_DNS_SA = ${DYN_DNS_SA}; DYN_DNS_SD = ${DYN_DNS_SD}"
    logMessage "DEBUG: DYN_SMB_NN = ${DYN_SMB_NN}; DYN_SMB_WG = ${DYN_SMB_WG}; DYN_SMB_WA = ${DYN_SMB_WA}"

# Set up the MAN_... variables to contain manual network settings

    set +e # "grep" will return error status (1) if no matches are found, so don't fail on individual errors

    if echo "${MAN_DNS_CONFIG}" | grep -q "DomainName" ; then
        readonly MAN_DNS_DN="$(trim "$( echo "${MAN_DNS_CONFIG}" | sed -e 's/^.*DomainName[^{]*{[[:space:]]*\([^}]*\)[[:space:]]*}.*$/\1/g' )")"
    else
        readonly MAN_DNS_DN="";
    fi
    if echo "${MAN_DNS_CONFIG}" | grep -q "ServerAddresses" ; then
        readonly MAN_DNS_SA="$(trim "$( echo "${MAN_DNS_CONFIG}" | sed -e 's/^.*ServerAddresses[^{]*{[[:space:]]*\([^}]*\)[[:space:]]*}.*$/\1/g' )")"
    else
        readonly MAN_DNS_SA="";
    fi
    if echo "${MAN_DNS_CONFIG}" | grep -q "SearchDomains" ; then
        readonly MAN_DNS_SD="$(trim "$( echo "${MAN_DNS_CONFIG}" | sed -e 's/^.*SearchDomains[^{]*{[[:space:]]*\([^}]*\)[[:space:]]*}.*$/\1/g' )")"
    else
        readonly MAN_DNS_SD="";
    fi
    if echo "${MAN_SMB_CONFIG}" | grep -q "NetBIOSName" ; then
        readonly MAN_SMB_NN="$(trim "$( echo "${MAN_SMB_CONFIG}" | sed -e 's/^.*NetBIOSName : \([^[:space:]]*\).*$/\1/g' )")"
    else
        readonly MAN_SMB_NN="";
    fi
    if echo "${MAN_SMB_CONFIG}" | grep -q "Workgroup" ; then
        readonly MAN_SMB_WG="$(trim "$( echo "${MAN_SMB_CONFIG}" | sed -e 's/^.*Workgroup : \([^[:space:]]*\).*$/\1/g' )")"
    else
        readonly MAN_SMB_WG="";
    fi
    if echo "${MAN_SMB_CONFIG}" | grep -q "WINSAddresses" ; then
        readonly MAN_SMB_WA="$(trim "$( echo "${MAN_SMB_CONFIG}" | sed -e 's/^.*WINSAddresses[^{]*{[[:space:]]*\([^}]*\)[[:space:]]*}.*$/\1/g' )")"
    else
        readonly MAN_SMB_WA="";
    fi

    set -e # resume abort on error

    logMessage "DEBUG:"
    logMessage "DEBUG: MAN_DNS_DN = ${MAN_DNS_DN}; MAN_DNS_SA = ${MAN_DNS_SA}; MAN_DNS_SD = ${MAN_DNS_SD}"
    logMessage "DEBUG: MAN_SMB_NN = ${MAN_SMB_NN}; MAN_SMB_WG = ${MAN_SMB_WG}; MAN_SMB_WA = ${MAN_SMB_WA}"
    
# Set up the CUR_... variables to contain the current network settings (from manual or DHCP, as arbitrated by OS X

    set +e # "grep" will return error status (1) if no matches are found, so don't fail on individual errors

    if echo "${CUR_DNS_CONFIG}" | grep -q "DomainName" ; then
        readonly CUR_DNS_DN="$(trim "$( echo "${CUR_DNS_CONFIG}" | sed -e 's/^.*DomainName[^{]*{[[:space:]]*\([^}]*\)[[:space:]]*}.*$/\1/g' )")"
    else
        readonly CUR_DNS_DN="";
    fi
    if echo "${CUR_DNS_CONFIG}" | grep -q "ServerAddresses" ; then
        readonly CUR_DNS_SA="$(trim "$( echo "${CUR_DNS_CONFIG}" | sed -e 's/^.*ServerAddresses[^{]*{[[:space:]]*\([^}]*\)[[:space:]]*}.*$/\1/g' )")"
    else
        readonly CUR_DNS_SA="";
    fi
    if echo "${CUR_DNS_CONFIG}" | grep -q "SearchDomains" ; then
        readonly CUR_DNS_SD="$(trim "$( echo "${CUR_DNS_CONFIG}" | sed -e 's/^.*SearchDomains[^{]*{[[:space:]]*\([^}]*\)[[:space:]]*}.*$/\1/g' )")"
    else
        readonly CUR_DNS_SD="";
    fi
    if echo "${CUR_SMB_CONFIG}" | grep -q "NetBIOSName" ; then
        readonly CUR_SMB_NN="$(trim "$( echo "${CUR_SMB_CONFIG}" | sed -e 's/^.*NetBIOSName : \([^[:space:]]*\).*$/\1/g' )")"
    else
        readonly CUR_SMB_NN="";
    fi
    if echo "${CUR_SMB_CONFIG}" | grep -q "Workgroup" ; then
        readonly CUR_SMB_WG="$(trim "$( echo "${CUR_SMB_CONFIG}" | sed -e 's/^.*Workgroup : \([^[:space:]]*\).*$/\1/g' )")"
    else
        readonly CUR_SMB_WG="";
    fi
    if echo "${CUR_SMB_CONFIG}" | grep -q "WINSAddresses" ; then
        readonly CUR_SMB_WA="$(trim "$( echo "${CUR_SMB_CONFIG}" | sed -e 's/^.*WINSAddresses[^{]*{[[:space:]]*\([^}]*\)[[:space:]]*}.*$/\1/g' )")"
    else
        readonly CUR_SMB_WA="";
    fi

    set -e # resume abort on error

    logMessage "DEBUG:"
    logMessage "DEBUG: CUR_DNS_DN = ${CUR_DNS_DN}; CUR_DNS_SA = ${CUR_DNS_SA}; CUR_DNS_SD = ${CUR_DNS_SD}"
    logMessage "DEBUG: CUR_SMB_NN = ${CUR_SMB_NN}; CUR_SMB_WG = ${CUR_SMB_WG}; CUR_SMB_WA = ${CUR_SMB_WA}"

    # set up the FIN_... variables with what we want to set things to

# Five FIN_... variables are simple -- no aggregation is done for them

    if [ "${DYN_DNS_DN}" != "" ] ; then
        if [ "${MAN_DNS_DN}" != "" ] ; then
            if "${ARG_OVERWRITE_NETCONF}" ; then
                readonly FIN_DNS_DN="${DYN_DNS_DN}"
            else
                logMessage "DomainName '$DYN_DNS_DN' ignored because DomainName was set manually and 'OVERWRITE_NETCONF' was not selected"
                readonly FIN_DNS_DN="${MAN_DNS_DN}"
            fi
        else
            readonly FIN_DNS_DN="${DYN_DNS_DN}"
        fi
    else
        readonly FIN_DNS_DN="${CUR_DNS_DN}"
    fi

    if [ "${DYN_SMB_NN}" != "" ] ; then
        if [ "${MAN_SMB_NN}" != "" ] ; then
            if "${ARG_OVERWRITE_NETCONF}" ; then
                readonly FIN_SMB_NN="${DYN_SMB_NN}"
            else
                logMessage "NetBIOSName '$DYN_SMB_NN' ignored because NetBIOSName was set manually and 'OVERWRITE_NETCONF' was not selected"
                readonly FIN_SMB_NN="${MAN_SMB_NN}"
            fi
        else
            readonly FIN_SMB_NN="${DYN_SMB_NN}"
        fi
    else
        readonly FIN_SMB_NN="${CUR_SMB_NN}"
    fi

    if [ "${DYN_SMB_WG}" != "" ] ; then
        if [ "${MAN_SMB_WG}" != "" ] ; then
            if "${ARG_OVERWRITE_NETCONF}" ; then
                readonly FIN_SMB_WG="${DYN_SMB_WG}"
            else
                logMessage "Workgroup '$DYN_SMB_WG' ignored because Workgroup was set manually and 'OVERWRITE_NETCONF' was not selected"
                readonly FIN_SMB_WG="${MAN_SMB_WG}"
            fi
        else
            readonly FIN_SMB_WG="${DYN_SMB_WG}"
        fi
    else
        readonly FIN_SMB_WG="${CUR_SMB_WG}"
    fi

    if [ "$DYN_DNS_SA" != "" ] ; then
        if [ "${MAN_DNS_SA}" != "" ] ; then
            if "${ARG_OVERWRITE_NETCONF}" ; then
                readonly FIN_DNS_SA="${DYN_DNS_SA}"
            else
                logMessage "ServerAddresses '$DYN_DNS_SA' ignored because ServerAddresses was set manually and 'OVERWRITE_NETCONF' was not selected"
                readonly FIN_DNS_SA="${MAN_DNS_SA}"
            fi
        else
            readonly FIN_DNS_SA="${DYN_DNS_SA}"
        fi
    else
        readonly FIN_DNS_SA="${CUR_DNS_SA}"
    fi

    if [ "$DYN_SMB_WA" != "" ] ; then
        if [ "${MAN_SMB_WA}" != "" ] ; then
            if "${ARG_OVERWRITE_NETCONF}" ; then
                readonly FIN_SMB_WA="${DYN_SMB_WA}"
            else
                logMessage "WINSAddresses '$DYN_SMB_WA' ignored because WINSAddresses was set manually and 'OVERWRITE_NETCONF' was not selected"
                readonly FIN_SMB_WA="${MAN_SMB_WA}"
            fi
        else
            readonly FIN_SMB_WA="${DYN_SMB_WA}"
        fi
    else
        readonly FIN_SMB_WA="${CUR_SMB_WA}"
    fi

    # DNS SearchDomains (FIN_DNS_SD) is treated specially
    #
    # OLD BEHAVIOR:
    #     if SearchDomains was not set manually, we set SearchDomains to the DomainName
    #     else
    #          In OS X 10.4-10.5, we add the DomainName to the end of any manual SearchDomains (unless it is already there)
    #          In OS X 10.6+, if SearchDomains was entered manually, we ignore the DomainName
    #                         else we set SearchDomains to the DomainName
    #
    # NEW BEHAVIOR (done if ARG_PREPEND_DOMAIN_NAME is "true"):
    #
    #     if SearchDomains was entered manually, we do nothing
    #     else we PREpend DomainName to the existing SearchDomains
    #
    #     This behavior is meant to behave like Linux with Network Manager and Windows

    if "${ARG_PREPEND_DOMAIN_NAME}" ; then
        if [ "${MAN_DNS_SD}" = "" ] ; then
            logMessage "Prepended '${FIN_DNS_DN}' to search domains '${CUR_DNS_SD}' because the search domains were not set manually and 'PREPEND_SEARCHDOMAIN' was selected"
            readonly FIN_DNS_SD="$(trim "${FIN_DNS_DN}" "${CUR_DNS_SD}")"
        else
            logMessage "Did not prepend '${FIN_DNS_DN}' to search domains '${CUR_DNS_SD}' because the search domains were set manually and 'PREPEND_SEARCHDOMAIN' was selected"
            readonly FIN_DNS_SD="${CUR_DNS_SD}"
        fi
    else
        if [ "${MAN_DNS_SD}" = "" ] ; then
            logMessage "Setting search domains to '${FIN_DNS_DN}' because the search domains were not set manually and 'PREPEND_SEARCHDOMAIN' was not selected"
            readonly FIN_DNS_SD="${FIN_DNS_DN}"
        else
            logMessage "Did not replace search domains '${CUR_DNS_SD}' with '${FIN_DNS_DN}' because the search domains were set manually and 'PREPEND_SEARCHDOMAIN' was not selected"
            readonly FIN_DNS_SD="${CUR_DNS_SD}"
        fi
    fi

    logMessage "DEBUG:"
    logMessage "DEBUG: FIN_DNS_DN = ${FIN_DNS_DN}; FIN_DNS_SA = ${FIN_DNS_SA}; FIN_DNS_SD = ${FIN_DNS_SD}"
    logMessage "DEBUG: FIN_SMB_NN = ${FIN_SMB_NN}; FIN_SMB_WG = ${FIN_SMB_WG}; FIN_SMB_WA = ${FIN_SMB_WA}"

# Set up SKP_... variables to inhibit scutil from making some changes

    # SKP_DNS_... and SKP_SMB_... are used to comment out individual items that are not being set
    if [ "${FIN_DNS_DN}" = "" -o "${FIN_DNS_DN}" = "${CUR_DNS_DN}" ] ; then
        SKP_DNS_DN="#"
    else
        SKP_DNS_DN=""
    fi
    if [ "${FIN_DNS_SA}" = "" -o "${FIN_DNS_SA}" = "${CUR_DNS_SA}" ] ; then
        SKP_DNS_SA="#"
    else
        SKP_DNS_SA=""
    fi
    if [ "${FIN_DNS_SD}" = "" -o "${FIN_DNS_SD}" = "${CUR_DNS_SD}" ] ; then
        SKP_DNS_SD="#"
    else
        SKP_DNS_SD=""
    fi
    if [ "${FIN_SMB_NN}" = "" -o "${FIN_SMB_NN}" = "${CUR_SMB_NN}" ] ; then
        SKP_SMB_NN="#"
    else
        SKP_SMBNN=""
    fi
    if [ "${FIN_SMB_WG}" = "" -o "${FIN_SMB_WG}" = "${CUR_SMB_WG}" ] ; then
        SKP_SMB_WG="#"
    else
        SKP_SMB_WG=""
    fi
    if [ "${FIN_SMB_WA}" = "" -o "${FIN_SMB_WA}" = "${CUR_SMB_WA}" ] ; then
        SKP_SMB_WA="#"
    else
        SKP_SMB_WA=""
    fi

    # if any DNS items should be set, set all that have values
    if [ "${SKP_DNS_DN}${SKP_DNS_SA}${SKP_DNS_SD}" = "###" ] ; then
        readonly SKP_DNS="#"
    else
        readonly SKP_DNS=""
        if [ "${FIN_DNS_DN}" != "" ] ; then
            SKP_DNS_DN=""
        fi
        if [ "${FIN_DNS_SA}" != "" ] ; then
            SKP_DNS_SA=""
        fi
        if [ "${FIN_DNS_SD}" != "" ] ; then
            SKP_DNS_SD=""
        fi
    fi

    # if any SMB items should be set, set all that have values
    if [ "${SKP_SMB_NN}${SKP_SMB_WG}${SKP_SMB_WA}" = "###" ] ; then
        readonly SKP_SMB="#"
    else
        readonly SKP_SMB=""
        if [ "${FIN_SMB_NN}" != "" ] ; then
            SKP_SMB_NN=""
        fi
        if [ "${FIN_SMB_WG}" != "" ] ; then
            SKP_SMB_WG=""
        fi
        if [ "${FIN_SMB_WA}" != "" ] ; then
            SKP_SMB_WA=""
        fi
    fi

    readonly SKP_DNS_SA SKP_DNS_SD SKP_DNS_DN
    readonly SKP_SMB_NN SKP_SMB_WG SKP_SMB_WA

# special-case fiddling:

    # in 10.8 and higher, ServerAddresses and SearchDomains must be set via the Setup: key in addition to the State: key
    # in 10.7 if ServerAddresses or SearchDomains are manually set, ServerAddresses and SearchDomains must be similarly set with the Setup: key in addition to the State: key
    #
    # we pass a flag indicating whether we've done that to the other scripts in 'bAlsoUsingSetupKeys'

    case "${OSVER}" in
        10.7 )
            if [ "${MAN_DNS_SA}" = "" -a "${MAN_DNS_SD}" = "" ] ; then
                logMessage "DEBUG: OS X 10.7 and neither ServerAddresses nor SearchDomains were set manually, so will modify DNS settings using only State:"
                readonly SKP_SETUP_DNS="#"
                readonly bAlsoUsingSetupKeys="false"
            else
                logMessage "DEBUG: OS X 10.7 and ServerAddresses or SearchDomains were set manually, so will modify DNS settings using Setup: in addition to State:"
                readonly SKP_SETUP_DNS=""
                readonly bAlsoUsingSetupKeys="true"
            fi
            ;;
        * )
            logMessage "DEBUG: OS X 10.8 or higher, so will modify DNS settings using Setup: in addition to State:"
            readonly SKP_SETUP_DNS=""
            readonly bAlsoUsingSetupKeys="true"
            ;;
    esac

    logMessage "DEBUG:"
    logMessage "DEBUG: SKP_DNS = ${SKP_DNS}; SKP_DNS_SA = ${SKP_DNS_SA}; SKP_DNS_SD = ${SKP_DNS_SD}; SKP_DNS_DN = ${SKP_DNS_DN}"
    logMessage "DEBUG: SKP_SETUP_DNS = ${SKP_SETUP_DNS}"
    logMessage "DEBUG: SKP_SMB = ${SKP_SMB}; SKP_SMB_NN = ${SKP_SMB_NN}; SKP_SMB_WG = ${SKP_SMB_WG}; SKP_SMB_WA = ${SKP_SMB_WA}"

    original_resolver_contents="`cat /etc/resolv.conf | grep -v '#'`"
    logMessage "DEBUG:"
    logMessage "DEBUG: /etc/resolve = ${original_resolver_contents}"
    logMessage "DEBUG:"

    scutil_dns="$( scutil --dns)"
    logMessage "DEBUG:"
    logMessage "DEBUG: scutil --dns BEFORE CHANGES = ${scutil_dns}"
    logMessage "DEBUG:"

    logMessage "DEBUG:"
    logMessage "DEBUG: Configuration changes:"
    logMessage "DEBUG: ${SKP_DNS}${SKP_DNS_SA}ADD State: ServerAddresses  ${FIN_DNS_SA}"
    logMessage "DEBUG: ${SKP_DNS}${SKP_DNS_SD}ADD State: SearchDomains    ${FIN_DNS_SD}"
    logMessage "DEBUG: ${SKP_DNS}${SKP_DNS_DN}ADD State: DomainName       ${FIN_DNS_DN}"
    logMessage "DEBUG:"
    logMessage "DEBUG: ${SKP_SETUP_DNS}${SKP_DNS}${SKP_DNS_SA}ADD Setup: ServerAddresses  ${FIN_DNS_SA}"
    logMessage "DEBUG: ${SKP_SETUP_DNS}${SKP_DNS}${SKP_DNS_SD}ADD Setup: SearchDomains    ${FIN_DNS_SD}"
    logMessage "DEBUG: ${SKP_SETUP_DNS}${SKP_DNS}${SKP_DNS_DN}ADD Setup: DomainName       ${FIN_DNS_DN}"
    logMessage "DEBUG:"
    logMessage "DEBUG: ${SKP_SMB}${SKP_SMB_NN}ADD State: NetBIOSName    ${FIN_SMB_NN}"
    logMessage "DEBUG: ${SKP_SMB}${SKP_SMB_WG}ADD State: Workgroup      ${FIN_SMB_WG}"
    logMessage "DEBUG: ${SKP_SMB}${SKP_SMB_WA}ADD State: WINSAddresses  ${FIN_SMB_WA}"

    # Save the openvpn process ID and the Network Primary Service ID, leasewather.plist path, logfile path, and optional arguments from Shimo,
    # then save old and new DNS and SMB settings
    # PPID is a script variable (defined by bash itself) that contains the process ID of the parent of the process running the script (i.e., Racoon's process ID)
    # config is an environmental variable set to the configuration path by Racoon prior to running this up script

    scutil <<-EOF > /dev/null
        open

        # Store our variables for the other scripts (leasewatch, down, etc.) to use
        d.init
        # The '#' in the next line does NOT start a comment; it indicates to scutil that a number follows it (as opposed to a string or an array)
        d.add PID # ${PPID}
        d.add Service ${PSID}
        d.add bAlsoUsingSetupKeys   "${bAlsoUsingSetupKeys}"
        set State:/Network/Racoon

        # Back up the device's current DNS and SMB configurations,
        # Indicate 'no such key' by a dictionary with a single entry: "ShimoNoSuchKey : true"
        # If there isn't a key, "ShimoNoSuchKey : true" won't be removed.
        # If there is a key, "ShimoNoSuchKey : true" will be removed and the key's contents will be used

        d.init
        d.add ShimoNoSuchKey true
        get State:/Network/Service/${PSID}/DNS
        set State:/Network/Racoon/OldDNS

        d.init
        d.add ShimoNoSuchKey true
        get Setup:/Network/Service/${PSID}/DNS
        set State:/Network/Racoon/OldDNSSetup

        d.init
        d.add ShimoNoSuchKey true
        get State:/Network/Service/${PSID}/SMB
        set State:/Network/Racoon/OldSMB

        # Initialize the new DNS map via State:
        ${SKP_DNS}d.init
        ${SKP_DNS}${SKP_DNS_SA}d.add ServerAddresses * ${FIN_DNS_SA}
        ${SKP_DNS}${SKP_DNS_SD}d.add SearchDomains   * ${FIN_DNS_SD}
        ${SKP_DNS}${SKP_DNS_DN}d.add DomainName        ${FIN_DNS_DN}
        ${SKP_DNS}set State:/Network/Service/${PSID}/DNS

        # If necessary, initialize the new DNS map via Setup: also
        ${SKP_SETUP_DNS}${SKP_DNS}d.init
        ${SKP_SETUP_DNS}${SKP_DNS}${SKP_DNS_SA}d.add ServerAddresses * ${FIN_DNS_SA}
        ${SKP_SETUP_DNS}${SKP_DNS}${SKP_DNS_SD}d.add SearchDomains   * ${FIN_DNS_SD}
        ${SKP_SETUP_DNS}${SKP_DNS}${SKP_DNS_DN}d.add DomainName        ${FIN_DNS_DN}
        ${SKP_SETUP_DNS}${SKP_DNS}set Setup:/Network/Service/${PSID}/DNS

        # Initialize the SMB map
        ${SKP_SMB}d.init
        ${SKP_SMB}${SKP_SMB_NN}d.add NetBIOSName     ${FIN_SMB_NN}
        ${SKP_SMB}${SKP_SMB_WG}d.add Workgroup       ${FIN_SMB_WG}
        ${SKP_SMB}${SKP_SMB_WA}d.add WINSAddresses * ${FIN_SMB_WA}
        ${SKP_SMB}set State:/Network/Service/${PSID}/SMB

        quit
EOF

    readonly NEW_DNS_SETUP_CONFIG="$( scutil <<-EOF |
        open
        show Setup:/Network/Service/${PSID}/DNS
        quit
EOF
sed -e 's/^[[:space:]]*[[:digit:]]* : //g' | tr '\n' ' '
)"
    readonly NEW_DNS_STATE_CONFIG="$( scutil <<-EOF |
        open
        show State:/Network/Service/${PSID}/DNS
        quit
EOF
sed -e 's/^[[:space:]]*[[:digit:]]* : //g' | tr '\n' ' '
)"
    readonly NEW_SMB_STATE_CONFIG="$( scutil <<-EOF |
        open
        show State:/Network/Service/${PSID}/SMB
        quit
EOF
sed -e 's/^[[:space:]]*[[:digit:]]* : //g' | tr '\n' ' '
)"
    readonly NEW_DNS_GLOBAL_CONFIG="$( scutil <<-EOF |
        open
        show State:/Network/Global/DNS
        quit
EOF
sed -e 's/^[[:space:]]*[[:digit:]]* : //g' | tr '\n' ' '
)"
    readonly NEW_SMB_GLOBAL_CONFIG="$( scutil <<-EOF |
        open
        show State:/Network/Global/SMB
        quit
EOF
sed -e 's/^[[:space:]]*[[:digit:]]* : //g' | tr '\n' ' '
)"

    logMessage "DEBUG:"
    logMessage "DEBUG: Configurations as read back after changes:"
    logMessage "DEBUG: State:/.../DNS = ${NEW_DNS_STATE_CONFIG}"
    logMessage "DEBUG: State:/.../SMB = ${NEW_SMB_STATE_CONFIG}"
    logMessage "DEBUG:"
    logMessage "DEBUG: Setup:/.../DNS = ${NEW_DNS_SETUP_CONFIG}"
    logMessage "DEBUG:"
    logMessage "DEBUG: State:/Network/Global/DNS = ${NEW_DNS_GLOBAL_CONFIG}"
    logMessage "DEBUG: State:/Network/Global/SMB = ${NEW_SMB_GLOBAL_CONFIG}"

    new_resolver_contents="`cat /etc/resolv.conf | grep -v '#'`"
    logMessage "DEBUG:"
    logMessage "DEBUG: /etc/resolve = ${new_resolver_contents}"
    logMessage "DEBUG:"

    scutil_dns="$( scutil --dns)"
    logMessage "DEBUG:"
    logMessage "DEBUG: scutil --dns AFTER CHANGES = ${scutil_dns}"
    logMessage "DEBUG:"

    logMessage "Saved the DNS and SMB configurations for later use"

}

restore_dns()
{
    # Quick check - is the configuration there?
    if ! scutil -w State:/Network/Racoon &>/dev/null -t 1 ; then
        # Configuration isn't there, so we forget it
        logMessage "WARNING: No existing Racoon DNS configuration found; not tearing down anything; exiting."
        return
    fi

    # Get info saved by the up script
    SHIMO_CONFIG="$( scutil <<-EOF
        open
        show State:/Network/Racoon
        quit
EOF)"
    
    PSID="$(echo "${SHIMO_CONFIG}" | grep -i '^[[:space:]]*Service :' | sed -e 's/^.*: //g')"
    bAlsoUsingSetupKeys="$(echo "${SHIMO_CONFIG}" | grep -i '^[[:space:]]*bAlsoUsingSetupKeys :' | sed -e 's/^.*: //g')"

    # Issue warning if the primary service ID has changed
    PSID_CURRENT="$( scutil <<-EOF |
        open
        show State:/Network/Racoon
        quit
EOF
grep Service | sed -e 's/.*Service : //'
)"
    if [ "${PSID}" != "${PSID_CURRENT}" ] ; then
        logMessage "Ignoring change of Network Primary Service from ${PSID} to ${PSID_CURRENT}"
    fi
    

    # Restore configurations
    DNS_OLD="$( scutil <<-EOF
        open
        show State:/Network/Racoon/OldDNS
        quit
EOF
)"
    SMB_OLD="$( scutil <<-EOF
        open
        show State:/Network/Racoon/OldSMB
        quit
EOF
)"
    DNS_OLD_SETUP="$( scutil <<-EOF
        open
        show State:/Network/Racoon/OldDNSSetup
        quit
EOF
)"
    CS_NO_SUCH_KEY="<dictionary> {
        ShimoNoSuchKey : true
    }"

    if [ "${DNS_OLD}" = "${CS_NO_SUCH_KEY}" ] ; then
        scutil <<-EOF
            open
            remove State:/Network/Service/${PSID}/DNS
            quit
EOF
    else
        scutil <<-EOF
            open
            get State:/Network/Racoon/OldDNS
            set State:/Network/Service/${PSID}/DNS
            quit
EOF
    fi

    if [ "${DNS_OLD_SETUP}" = "${CS_NO_SUCH_KEY}" ] ; then
        if ${bAlsoUsingSetupKeys} ; then
            logMessage "DEBUG: Removing 'Setup:' DNS key"
            scutil <<-EOF
                open
                remove Setup:/Network/Service/${PSID}/DNS
                quit
EOF
        else
            logMessage "DEBUG: Not removing 'Setup:' DNS key"
        fi
    else
        if ${bAlsoUsingSetupKeys} ; then
            logMessage "DEBUG: Restoring 'Setup:' DNS key"
            scutil <<-EOF
                open
                get State:/Network/Racoon/OldDNSSetup
                set Setup:/Network/Service/${PSID}/DNS
                quit
EOF
        else
            logMessage "DEBUG: Not restoring 'Setup:' DNS key"
        fi
    fi

    if [ "${SMB_OLD}" = "${CS_NO_SUCH_KEY}" ] ; then
        scutil <<-EOF
            open
            remove State:/Network/Service/${PSID}/SMB
            quit
EOF
    else
        scutil <<-EOF
            open
            get State:/Network/Racoon/OldSMB
            set State:/Network/Service/${PSID}/SMB
            quit
EOF
    fi

    logMessage "Restored the DNS and SMB configurations"

    # Remove our system configuration data
    scutil <<-EOF
        open
        remove State:/Network/Racoon/OldDNS
        remove State:/Network/Racoon/OldSMB
        remove State:/Network/Racoon/OldDNSSetup
        remove State:/Network/Racoon
        quit
EOF
}

add_spd()
{
    # update SA database
    LOCAL="${LOCAL_ADDR}"
    REMOTE="${REMOTE_ADDR}"
    if [ "x${LOCAL_PORT}" != "x500" ]; then
        # NAT-T setup
        LOCAL="${LOCAL}[${LOCAL_PORT}]"
        REMOTE="${REMOTE}[${REMOTE_PORT}]"
    fi

    echo "spdadd ${INTERNAL_ADDR4}/32[any] 0.0.0.0/0[any] any -P out ipsec esp/tunnel/${LOCAL}-${REMOTE}/require;\
spdadd 0.0.0.0/0[any] ${INTERNAL_ADDR4}[any] any -P in ipsec esp/tunnel/${REMOTE}-${LOCAL}/require;\
" | setkey -c
}

del_spd()
{
    # clean up SA database
    LOCAL="${LOCAL_ADDR}"
    REMOTE="${REMOTE_ADDR}"
    if [ "x${LOCAL_PORT}" != "x500" ]; then
        # NAT-T setup
        LOCAL="${LOCAL}[${LOCAL_PORT}]"
        REMOTE="${REMOTE}[${REMOTE_PORT}]"
    fi

    echo "deleteall ${REMOTE_ADDR} ${LOCAL_ADDR} esp;\
deleteall ${LOCAL_ADDR} ${REMOTE_ADDR} esp;\
spddelete ${INTERNAL_ADDR4}/32[any] 0.0.0.0/0[any] any -P out ipsec esp/tunnel/${LOCAL}-${REMOTE}/require;\
spddelete 0.0.0.0/0[any] ${INTERNAL_ADDR4}[any] any -P in ipsec esp/tunnel/${REMOTE}-${LOCAL}/require;\
" | setkey -c
}

##########################################################################################
flushDNSCache()
{
    if ${ARG_FLUSH_DNS_CACHE} ; then
        case "${OSVER}" in
            * )
                set +e # "grep" will return error status (1) if no matches are found, so don't fail on individual errors
                hands_off_ps="$( ps -ax | grep HandsOffDaemon | grep -v grep.HandsOffDaemon )"
                set -e # We instruct bash that it CAN again fail on errors
                if [ "${hands_off_ps}" = "" ] ; then
                    if [ -f /usr/bin/killall ] ; then
                        /usr/bin/killall -HUP mDNSResponder
                        logMessage "Flushed the DNS Cache"
                    else
                        logMessage "/usr/bin/killall not present. Not flushing the DNS cache"
                    fi
                else
                    logMessage "Hands Off is running. Not flushing the DNS cache"
                fi
                ;;
        esac
    fi
}


### Main Functions

# bring up phase1
phase1_up()
{
    # check if VPN address already set up on default interface (dupe script call)
    ifconfig ${INTERFACE} | grep -q "${INTERNAL_ADDR4}" && {
        logMessage "p1_up_down: phase1_up has already run !!!"
        exit 4
    }

    # update MacOS DNS
    if [ -n "${INTERNAL_DNS4_LIST}" ]; then
        modify_dns
        flushDNSCache
    fi

    if [ -n "${INTERNAL_NETMASK4}" ]; then
        NETMASK=${INTERNAL_NETMASK4}
    else
        NETMASK="255.255.255.255"
    fi

    # add VPN address to default interface
    ifconfig ${INTERFACE} alias ${INTERNAL_ADDR4} netmask ${NETMASK}
    # set up host route to VPN server
    route add ${REMOTE_ADDR} ${DEFAULT_GW}

    if [ -n "${SPLIT_INCLUDE}" ]; then
        logMessage "using split tunneling with received split routes"
        # split tunnel: keep existing default, insert specific tunnel routes
        for N in ${SPLIT_INCLUDE}; do
            route add ${N} ${INTERNAL_ADDR4}
        done

        if "${ARG_DEFAULT_ROUTE}" ; then
            logMessage "replacing default route as ARG_DEFAULT_ROUTE is set"

            # save old default gateway
            echo "$DEFAULT_GW" > "${DEFAULT_ROUTE_BACKUP}"

            # replace default route with vpn tunnel
            route delete default
            route add default ${INTERNAL_ADDR4}
        fi
    else
        logMessage "set up local route exceptions"

        # full tunnel: set up any applicable exceptions...
        for N in ${SPLIT_LOCAL}; do
            if [[ ${N} == *"0.0.0.0"* ]]; then
                SKIP_DEFAULT_ROUTE="true"
            else
                route add ${N} ${DEFAULT_GW}
            fi
        done
        
        if [ -z ${SKIP_DEFAULT_ROUTE} ] || "${ARG_DEFAULT_ROUTE}" ; then
            logMessage "replacing default route"
            
            # save old default gateway
            echo "$DEFAULT_GW" > "${DEFAULT_ROUTE_BACKUP}"

            # ... then replace default route with vpn tunnel
            route delete default
            route add default ${INTERNAL_ADDR4}
        elif [ -n ${SKIP_DEFAULT_ROUTE} ]; then
            logMessage "do not replace default route as server includes default route in split local"
        fi
    fi

    # add security policies
    add_spd
}

# bring down phase1
phase1_down() 
{
    # restore MacOS DNS
    restore_dns
    flushDNSCache

    if [ -n "${SPLIT_INCLUDE}" ]; then
        # split tunnel: remove specific tunnel routes
        for N in ${SPLIT_INCLUDE}; do
            route delete ${N}
        done

        if "${ARG_DEFAULT_ROUTE}" ; then
            # restore default route from file
            DEFAULT_GW=`cat "${DEFAULT_ROUTE_BACKUP}"`

            # restore original default route
            route delete default
            route add default ${DEFAULT_GW}
        fi
    else
        # full tunnel: remove any applicable exceptions...
        for N in ${SPLIT_LOCAL}; do
            if [[ ${N} == *"0.0.0.0"* ]]; then
                logMessage "do not reset default route as server included default route in split local"
            else
                route delete ${N}
            fi
        done

        if [ -s "${DEFAULT_ROUTE_BACKUP}" ]; then
            DEFAULT_GW=`cat "${DEFAULT_ROUTE_BACKUP}"`

            # ... then restore original default route
            route delete default
            route add default ${DEFAULT_GW}
        fi
    fi

    # remove host route to VPN server
    route delete ${REMOTE_ADDR}
    # remove VPN address from default interface
    ifconfig ${INTERFACE} delete ${INTERNAL_ADDR4}
    
    # delete security policies
    del_spd
    
    # reset primary interface, if wanted
    reset_interface
}

reset_interface()
{ 
    if ${ARG_RESET_PRIMARY_INTERFACE_ON_DISCONNECT} ; then

        set +e # "grep" will return error status (1) if no matches are found, so don't fail if not found
        PINTERFACE="$( scutil <<-EOF |
        open
        show State:/Network/Global/IPv4
        quit
EOF
        grep PrimaryInterface | sed -e 's/.*PrimaryInterface : //'
    )"
        set -e # resume abort on error

        if [ "${PINTERFACE}" != "" ] ; then
            logMessage "Resetting primary interface '${PINTERFACE}'..."
            /sbin/ifconfig "${PINTERFACE}" down
            /sbin/ifconfig "${PINTERFACE}" up
        else
            logMessage "Not resetting primary interface because it cannot be found."
        fi

    fi
}

##########################################################################################
#
# START OF SCRIPT
#
##########################################################################################
trap "" TSTP
trap "" HUP
trap "" INT
export PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin

readonly SCRIPT_FILE=$0
readonly SCRIPT_LOG_FILE="${SCRIPT_FILE%%.*}.log"
readonly SCRIPT_PATH="${0%/*}"

readonly DEFAULT_ROUTE_BACKUP="${SCRIPT_PATH}/racoon_defaultroute_${REMOTE_ADDR}.old"

# process parameters to script.
# set defaults, if not already set before
if [ -z ${ARG_PREPEND_DOMAIN_NAME} ]; then
    ARG_PREPEND_DOMAIN_NAME="false"
fi
if [ -z ${ARG_FLUSH_DNS_CACHE} ]; then
    ARG_FLUSH_DNS_CACHE="false"
fi
if [ -z ${ARG_OVERWRITE_NETCONF} ]; then
    ARG_OVERWRITE_NETCONF="false"
fi
if [ -z ${ARG_DEFAULT_ROUTE} ]; then
    ARG_DEFAULT_ROUTE="false"
fi
if [ -z ${ARG_RESET_PRIMARY_INTERFACE_ON_DISCONNECT} ]; then
    ARG_RESET_PRIMARY_INTERFACE_ON_DISCONNECT="false"
fi

readonly ARG_PREPEND_DOMAIN_NAME ARG_FLUSH_DNS_CACHE ARG_OVERWRITE_NETCONF ARG_DEFAULT_ROUTE ARG_RESET_PRIMARY_INTERFACE_ON_DISCONNECT

DEFAULT_GW=`netstat -r -n | awk '/:|link/ { next; } /^(default|0\.0\.0\.0)/ { print $2; }'`
INTERFACE=`netstat -finet -rn | awk '($1 == "default"){print $6; exit}'`

readonly REASON=$1

readonly OSVER="$(sw_vers | grep 'ProductVersion:' | grep -o '10\.[0-9]*')"

if [ -z "$REASON" ]; then
    echo "this script must be called from racoon"
    exit 1
fi

# check for valid VPN address
echo ${INTERNAL_ADDR4} | grep -q '[0-9]' || {
    logMessage "p1_up_down: error: invalid INTERNAL_ADDR4."
    exit 1
}

logMessage "$REASON starting..."

# print available parameters
print_variables

chmod 644 ${SCRIPT_LOG_FILE}

case "$REASON" in
    phase1_up)
        phase1_up
        ;;
    phase1_down)
        phase1_down
        ;;
    *)
        echo "unknown reason '$REASON'. Maybe racoon-script is out of date"
        exit 1
        ;;
esac

logMessage "$REASON completed successfully."
exit 0


